<?php

/**
 * @file
 * Contains the actual compilation functions.
 */

/**
 * Main compilation function, run from the cron function.
 */
function closure_compiler_compile() {
  module_load_include('inc', 'closure_compiler', 'closure_compiler.defines');

  // Ensure the directory exists and is writeable.
  $js_dir = CLOSURE_COMPILER_JS_PATH;
  if (!file_prepare_directory($js_dir, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
    watchdog('closure_compiler', '%path directory not available', array('%path' => CLOSURE_COMPILER_JS_PATH), WATCHDOG_WARNING);
    return;
  }

  // Create list of files to compile.
  module_load_include('inc', 'closure_compiler', 'closure_compiler.functions');
  $js_files = closure_compiler_get_js_files();
  if (empty($js_files['unoptimized'])) {
    if (empty($js_files['optimized'])) {
      watchdog('closure_compiler', 'No JavaScript files found at %path', array('%path' => CLOSURE_COMPILER_JS_PATH), WATCHDOG_NOTICE);
    }
    else {
      watchdog('closure_compiler', 'No unoptimized JavaScript files found at %path', array('%path' => CLOSURE_COMPILER_JS_PATH), WATCHDOG_INFO);
    }
    return;
  }

  // Proceed to optimize the JS files.
  $process_limit = variable_get('closure_compiler_process_limit', CLOSURE_COMPILER_DEFAULT_LIMIT);
  $compilation_level = variable_get('closure_compiler_compilation_level', CLOSURE_COMPILER_SIMPLE);
  $process_method = variable_get('closure_compiler_process_method', CLOSURE_COMPILER_SEND_CONTENTS);
  $closure_compiler_processed = variable_get('closure_compiler_processed', array());
  $succeeded = $failed = 0;
  foreach ($js_files['unoptimized'] as $uri => $js_file) {
    // Stop processing if limit is reached.
    if ($process_limit && ($succeeded + $failed) == $process_limit) {
      break;
    }

    // Attempt to optimize an individual file.
    $source_uri = closure_compiler_get_source_uri($uri);
    if (!file_unmanaged_copy($uri, $source_uri, FILE_EXISTS_REPLACE) || !closure_compiler_compile_file($uri, $source_uri, $compilation_level, $process_method)) {
      ++$failed;
    }
    else {
      ++$succeeded;
    }
  }

  if (($succeeded + $failed) > 0) {
    watchdog('closure_compiler', 'Successfully compiled @succeeded javascript files, @failed failed.', array('@succeeded' => $succeeded, '@failed' => $failed), WATCHDOG_INFO);
  }
}

/**
 * Compiles a single JS file.
 *
 * @param string $uri
 *   The location of the JS file to compile.
 * @param string $source_uri
 *   The location to which the source link should point. If the source should
 *   not be available, then pass the boolean value FALSE.
 * @param string $compilation_level
 *   The compilation level to use (see closure_compiler.defines.inc).
 * @param int $process_method
 *   The process method to use (see closure_compiler.defines.inc).
 *
 * @return bool
 *   Whether the compilation was successful.
 */
function closure_compiler_compile_file($uri, $source_uri, $compilation_level, $process_method) {
  module_load_include('inc', 'closure_compiler', 'closure_compiler.defines');

  $result = FALSE;
  $source_url = $source_uri ? file_create_url($source_uri) : 'Unavailable';
  if (CLOSURE_COMPILER_LOCAL == $process_method) {
    $result = closure_compiler_compile_file_local($uri, $source_url, $compilation_level);
  }
  else {
    $params['compilation_level'] = $compilation_level;
    if (CLOSURE_COMPILER_SEND_CONTENTS == $process_method && filesize($uri) <= 200000) {
      $params['js_code'] = file_get_contents($uri);
    }
    else {
      $params['code_url'] = file_create_url($uri);
    }
    $result = closure_compiler_compile_file_api($uri, $source_url, $params);
  }
  if ($result) {
    // Replace GZip-compressed file if necessary.
    if (variable_get('js_gzip_compression', TRUE) && variable_get('clean_url', 0) && extension_loaded('zlib')) {
      file_unmanaged_save_data(gzencode(file_get_contents($uri), 9, FORCE_GZIP), $uri . '.gz', FILE_EXISTS_REPLACE);
    }
    elseif (file_exists($uri . '.gz')) {
      file_unmanaged_delete($uri . '.gz');
    }
  }
  return $result;
}

/**
 * Compiles a JS file locally.
 *
 * @param string $uri
 *   The location of the JS file to compile.
 * @param string $source_url
 *   The URL to which the source link should point.
 * @param string $compilation_level
 *   The compilation level to use (available options are defined).
 *
 * @return bool
 *   Whether the compilation was successful.
 */
function closure_compiler_compile_file_local($uri, $source_url, $compilation_level) {
  module_load_include('inc', 'closure_compiler', 'closure_compiler.defines');

  // The compiler script writes to a temporary file, prepends the signature
  // and replaces the original javascript file - the raw console output
  // can include error or warning messages, which breaks JS.
  $temp_file_path = drupal_tempnam(file_directory_temp(), 'jscc');
  $compiler_path = drupal_get_path('module', 'closure_compiler') . '/' . CLOSURE_COMPILER_FILE;
  $command = 'java -jar ' . $compiler_path . ' --compilation_level ' . $compilation_level . ' --js "' . drupal_realpath(check_plain($uri)) . '" --js_output_file ' . $temp_file_path;
  shell_exec($command);

  // Check whether the compiled file contains any content.
  $compiled_js = file_get_contents($temp_file_path);
  if (!$compiled_js) {
    return FALSE;
  }

  // Overwrite the original file.
  if (!file_unmanaged_save_data(CLOSURE_COMPILER_SOURCE_START . $source_url . CLOSURE_COMPILER_SOURCE_END . $compiled_js, $uri, FILE_EXISTS_REPLACE)) {
    return FALSE;
  }

  // Cleanup.
  file_unmanaged_delete($temp_file_path);
  return TRUE;
}

/**
 * Compiles a JS file using the API.
 *
 * @param string $uri
 *   The location of the JS file to compile.
 * @param string $source_url
 *   The URL to which the source link should point.
 * @param array $params
 *   Containing 'code_url' or 'js_code', and 'compilation_level'.
 *
 * @return bool
 *   Whether the compilation was successful.
 */
function closure_compiler_compile_file_api($uri, $source_url, $params) {
  module_load_include('inc', 'closure_compiler', 'closure_compiler.defines');

  // Perform the API request.
  $params['output_info'] = 'compiled_code';
  $params['output_format'] = 'json';
  $response = drupal_http_request(CLOSURE_COMPILER_JS_OPTIMIZATION_URL, array(
    'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
    'method' => 'POST',
    'data' => http_build_query($params, '', '&'),
  ));

  // Check the response is valid.
  if (200 != $response->code || !$response->data) {
    watchdog('closure_compiler', 'Request error: @request', array('@request' => print_r($response, TRUE)));
    return FALSE;
  }

  // Decode json in the response.
  $compiled_code = json_decode($response->data, TRUE);
  if (!$compiled_code || !is_array($compiled_code)) {
    watchdog('closure_compiler', 'Failed to parse JSON response: @response', array('@response' => $response->data));
    return FALSE;
  }

  // Check for server errors.
  if (array_key_exists('serverErrors', $compiled_code)) {
    watchdog('closure_compiler', 'Server error: @error', array('@error' => print_r($compiled_code['serverErrors'], TRUE)));
    return FALSE;
  }

  // Finally check the actual compiled code!
  if (array_key_exists('compiledCode', $compiled_code) && !empty($compiled_code['compiledCode'])) {
    // Success, write it back to the file.
    if (file_unmanaged_save_data(CLOSURE_COMPILER_SOURCE_START . $source_url . CLOSURE_COMPILER_SOURCE_END . $compiled_code['compiledCode'], $uri, FILE_EXISTS_REPLACE)) {
      return TRUE;
    }
    // Failed to write the compiled code back to the JS file.
    else {
      return FALSE;
    }
  }
  // The code is not present, or is empty.
  else {
    return FALSE;
  }
}
